/*
 *   (c) Yury Kuchura
 *   kuchura@gmail.com
 *
 *   This code can be used on terms of WTFPL Version 2 (http://www.wtfpl.net/).
 */

#include <stdint.h>
#include <stdio.h>
#include "LCD.h"
#include "dsp.h"
#include "font.h"
#include "touch.h"
#include "gen.h"
#include "config.h"
#include "hit.h"
#include "main.h"
#include "num_keypad.h"
#include "screenshot.h"


static uint32_t fChanged = 0;
static uint32_t rqExit = 0;
static uint32_t f_maxstep = 500000;
static uint32_t redrawWindow = 0;

void Sleep(uint32_t ms);

static void ShowF()
{
    FONT_ClearLine(FONT_FRANBIG, LCD_BLACK, 0);
    FONT_Print(FONT_FRANBIG, LCD_WHITE, LCD_RGB(31,31,127), 180, 00, " %u kHz ", (unsigned int)(CFG_GetParam(CFG_PARAM_GEN_F) / 1000));  //lpg
}

static void GENERATOR_SwitchWindow(void)
{
    rqExit = 1;
}

static void FDecr(uint32_t step)
{
    uint32_t MeasurementFreq = CFG_GetParam(CFG_PARAM_GEN_F);
    if(MeasurementFreq > step && MeasurementFreq % step != 0)
    {
        MeasurementFreq -= (MeasurementFreq % step);
        CFG_SetParam(CFG_PARAM_GEN_F, MeasurementFreq);
        fChanged = 1;
    }
    if(MeasurementFreq < BAND_FMIN)
    {
        MeasurementFreq = BAND_FMIN;
        CFG_SetParam(CFG_PARAM_GEN_F, MeasurementFreq);
        fChanged = 1;
    }
    if(MeasurementFreq > BAND_FMIN)
    {
        if(MeasurementFreq > step && (MeasurementFreq - step) >= BAND_FMIN)
        {
            MeasurementFreq = MeasurementFreq - step;
            CFG_SetParam(CFG_PARAM_GEN_F, MeasurementFreq);
            fChanged = 1;
        }
    }
}

static void FIncr(uint32_t step)
{
    uint32_t MeasurementFreq = CFG_GetParam(CFG_PARAM_GEN_F);
    if(MeasurementFreq > step && MeasurementFreq % step != 0)
    {
        MeasurementFreq -= (MeasurementFreq % step);
        CFG_SetParam(CFG_PARAM_GEN_F, MeasurementFreq);
        fChanged = 1;
    }
    if(MeasurementFreq < BAND_FMIN)
    {
        MeasurementFreq = BAND_FMIN;
        CFG_SetParam(CFG_PARAM_GEN_F, MeasurementFreq);
        fChanged = 1;
    }
    if(MeasurementFreq < CFG_GetParam(CFG_PARAM_BAND_FMAX))
    {
        if ((MeasurementFreq + step) > CFG_GetParam(CFG_PARAM_BAND_FMAX))
            MeasurementFreq = CFG_GetParam(CFG_PARAM_BAND_FMAX);
        else
            MeasurementFreq = MeasurementFreq + step;
        CFG_SetParam(CFG_PARAM_GEN_F, MeasurementFreq);
        fChanged = 1;
    }
}

static void GENERATOR_FDecr_500k(void)
{
    FDecr(f_maxstep);
}
static void GENERATOR_FDecr_100k(void)
{
    FDecr(100000);
}
static void GENERATOR_FDecr_5k(void)
{
    FDecr(5000);
}
static void GENERATOR_FIncr_5k(void)
{
    FIncr(5000);
}
static void GENERATOR_FIncr_100k(void)
{
    FIncr(100000);
}
static void GENERATOR_FIncr_500k(void)
{
    FIncr(f_maxstep);
}

static void GENERATOR_SetFreq(void)
{
    while(TOUCH_IsPressed());
    uint32_t val = NumKeypad(CFG_GetParam(CFG_PARAM_GEN_F)/1000, BAND_FMIN/1000, CFG_GetParam(CFG_PARAM_BAND_FMAX)/1000, "Set generator frequency, kHz");
    CFG_SetParam(CFG_PARAM_GEN_F, val * 1000);
    CFG_Flush();
    redrawWindow = 1;
}

static void GENERATOR_Screenshot(void)
{
    char* fname = 0;
    fname = SCREENSHOT_SelectFileName();
    SCREENSHOT_DeleteOldest();
    if (CFG_GetParam(CFG_PARAM_SCREENSHOT_FORMAT))
        SCREENSHOT_SavePNG(fname);
    else
        SCREENSHOT_Save(fname);
}

static const struct HitRect hitArr[] =
{
    //        x0,  y0, width, height, callback
    HITRECT(   0, 230, 100, 39, GENERATOR_SwitchWindow),
    HITRECT( 180,   0, 120, 39, GENERATOR_SetFreq),        //lpg  75, 200, 100,  79,
    HITRECT( 190, 230, 150, 39, GENERATOR_Screenshot),
    HITRECT(   0,   60, 80, 39, GENERATOR_FDecr_500k),
    HITRECT(  80,   60, 80, 39, GENERATOR_FDecr_100k),
    HITRECT( 160,   60, 70, 39, GENERATOR_FDecr_5k),
    HITRECT( 250,   60, 70, 39, GENERATOR_FIncr_5k),
    HITRECT( 320,   60, 80, 39, GENERATOR_FIncr_100k),
    HITRECT( 400,   60, 80, 39, GENERATOR_FIncr_500k),
    HITEND
};


void GENERATOR_Window_Proc(void)
{
    {
        uint32_t fbkup = CFG_GetParam(CFG_PARAM_GEN_F);
        if (!(fbkup >= BAND_FMIN && fbkup <= CFG_GetParam(CFG_PARAM_BAND_FMAX) && (fbkup % 1000) == 0))
        {
            CFG_SetParam(CFG_PARAM_GEN_F, 14000000);
            CFG_Flush();
        }
    }

    BSP_LCD_SelectLayer(0);
    LCD_FillAll(LCD_BLACK);
    BSP_LCD_SelectLayer(1);
    LCD_FillAll(LCD_BLACK);

    LCD_ShowActiveLayerOnly();

GENERATOR_REDRAW:
    while(TOUCH_IsPressed());

    GEN_SetMeasurementFreq(CFG_GetParam(CFG_PARAM_GEN_F));

    for (uint32_t layer = 0; layer < 2; layer++)
    {
        BSP_LCD_SelectLayer(layer);
        LCD_FillAll(LCD_BLACK);
//        FONT_Write(FONT_FRANBIG, LCD_WHITE, LCD_BLACK, 120, 2, "Generator mode");

        //Draw freq change areas bar
        FONT_Write(FONT_FRAN, LCD_WHITE, LCD_RGB(15,15,63) , 0,   60, "  -500k  ");
        FONT_Write(FONT_FRAN, LCD_WHITE, LCD_RGB(31,31,127), 80,  60, "  -100k  ");
        FONT_Write(FONT_FRAN, LCD_WHITE, LCD_RGB(64,64,255), 160, 60, "   -5k   ");
        FONT_Write(FONT_FRAN, LCD_WHITE, LCD_RGB(64,64,255), 240, 60, "   +5k   ");
        FONT_Write(FONT_FRAN, LCD_WHITE, LCD_RGB(31,31,127), 320, 60, "  +100k  ");
        FONT_Write(FONT_FRAN, LCD_WHITE, LCD_RGB(15,15,63) , 400, 60, "  +500k  ");
        FONT_Write(FONT_FRANBIG, LCD_WHITE, LCD_DYELLOW, 0, 230, "    Exit    ");
        FONT_Write(FONT_FRANBIG, LCD_WHITE, LCD_DBLUE, 190, 230, " Save snapshot ");
        ShowF();
    }

    uint32_t speedcnt = 0;
    f_maxstep = 500000;
    rqExit = 0;
    fChanged = 0;
    redrawWindow = 0;

    while(1)
    {
        LCDPoint pt;
        while (TOUCH_Poll(&pt))
        {
            HitTest(hitArr, pt.x, pt.y);
            if (rqExit)
            {
                GEN_SetMeasurementFreq(0);
                while(TOUCH_IsPressed());
                return; //Change window.. EXIT
            }
            if (redrawWindow)
            {
                goto GENERATOR_REDRAW;
            }
            if (fChanged)
            {
                ShowF();
            }
            speedcnt++;
            if (speedcnt < 20)
                Sleep(100);
            else
                Sleep(30);
            if (speedcnt > 50)
                f_maxstep = 2000000;
        }
        speedcnt = 0;
        f_maxstep = 500000;
        if (fChanged)
        {
            CFG_Flush();
            GEN_SetMeasurementFreq(CFG_GetParam(CFG_PARAM_GEN_F));
            fChanged = 0;
        }

        //Measure without changing current frequency and without OSL
        DSP_Measure(0, 1, 0, CFG_GetParam(CFG_PARAM_MEAS_NSCANS));

        uint32_t activeLayer = BSP_LCD_GetActiveLayer();
        BSP_LCD_SelectLayer(!activeLayer);

        ShowF();
        FONT_SetAttributes(FONT_FRAN, LCD_WHITE, LCD_BLACK);

        FONT_ClearLine(FONT_FRAN, LCD_BLACK, 100);
        FONT_Printf(0, 100, "Raw Vi: %.2f mV, Vv: %.2f mV Diff %.2f dB", DSP_MeasuredMagImv(),
                     DSP_MeasuredMagVmv(), DSP_MeasuredDiffdB());

        FONT_ClearLine(FONT_FRAN, LCD_BLACK, 120);
        FONT_Printf(0, 120, "Raw phase diff: %.1f deg", DSP_MeasuredPhaseDeg());

        if (DSP_MeasuredMagVmv() < 1.f)
            FONT_Write(FONT_FRAN, LCD_BLACK, LCD_RED, 0, 140,   "No signal  ");
        else
        {
            FONT_Write(FONT_FRAN, LCD_GREEN, LCD_BLACK, 0, 140, "Signal OK   ");
            static DSP_RX rx;
            DSP_Measure(0, 1, 0, CFG_GetParam(CFG_PARAM_MEAS_NSCANS)); //without osl
            rx = DSP_MeasuredZ();
            FONT_ClearLine(FONT_FRAN, LCD_BLACK, 160);
            FONT_Printf(0, 160, "Raw     : %.2f  %.2fj", crealf(rx), cimagf(rx));

            DSP_Measure(0, 1, 1, CFG_GetParam(CFG_PARAM_MEAS_NSCANS)); //with osl
            rx = DSP_MeasuredZ();
            FONT_ClearLine(FONT_FRAN, LCD_BLACK, 180);
            FONT_Printf(0, 180, "With OSL: %.2f  %.2fj", crealf(rx), cimagf(rx));

            float VSWR = DSP_CalcVSWR(rx);
            FONT_ClearLine(FONT_FRAN, LCD_BLACK, 200);
            FONT_Printf(0, 200, "VSWR: %.2f for Z0: %lu ohms", VSWR, CFG_GetParam(CFG_PARAM_R0));
        }
        LCD_ShowActiveLayerOnly();
        Sleep(100);
    }
}
